
// This file has been generated by the GUI designer. Do not modify.
namespace MonoDevelop.Database.Components
{
	public partial class WaitDialog
	{
		private global::Gtk.VBox vbox;

		private global::Gtk.HBox hbox2;

		private global::Gtk.Image image;

		private global::Gtk.Label label;

		private global::Gtk.ProgressBar progressbar;

		protected virtual void Build ()
		{
			global::Stetic.Gui.Initialize (this);
			// Widget MonoDevelop.Database.Components.WaitDialog
			this.Name = "MonoDevelop.Database.Components.WaitDialog";
			this.Title = global::MonoDevelop.Database.AddinCatalog.GetString ("WaitDialog");
			this.TypeHint = ((global::Gdk.WindowTypeHint)(6));
			this.WindowPosition = ((global::Gtk.WindowPosition)(1));
			this.Modal = true;
			this.BorderWidth = ((uint)(1));
			this.Resizable = false;
			this.AllowGrow = false;
			this.Decorated = false;
			this.SkipTaskbarHint = true;
			// Container child MonoDevelop.Database.Components.WaitDialog.Gtk.Container+ContainerChild
			this.vbox = new global::Gtk.VBox ();
			this.vbox.Name = "vbox";
			this.vbox.Spacing = 6;
			this.vbox.BorderWidth = ((uint)(6));
			// Container child vbox.Gtk.Box+BoxChild
			this.hbox2 = new global::Gtk.HBox ();
			this.hbox2.Name = "hbox2";
			this.hbox2.Spacing = 6;
			// Container child hbox2.Gtk.Box+BoxChild
			this.image = new global::Gtk.Image ();
			this.image.Name = "image";
			this.image.Xalign = 0f;
			this.image.Yalign = 0f;
			this.image.Pixbuf = global::Stetic.IconLoader.LoadIcon (this, "stock_alarm", global::Gtk.IconSize.Dialog);
			this.hbox2.Add (this.image);
			global::Gtk.Box.BoxChild w1 = ((global::Gtk.Box.BoxChild)(this.hbox2[this.image]));
			w1.Position = 0;
			w1.Expand = false;
			w1.Fill = false;
			// Container child hbox2.Gtk.Box+BoxChild
			this.label = new global::Gtk.Label ();
			this.label.Name = "label";
			this.label.Xalign = 0f;
			this.label.Yalign = 0f;
			this.label.LabelProp = global::MonoDevelop.Database.AddinCatalog.GetString ("Please wait");
			this.hbox2.Add (this.label);
			global::Gtk.Box.BoxChild w2 = ((global::Gtk.Box.BoxChild)(this.hbox2[this.label]));
			w2.Position = 1;
			this.vbox.Add (this.hbox2);
			global::Gtk.Box.BoxChild w3 = ((global::Gtk.Box.BoxChild)(this.vbox[this.hbox2]));
			w3.Position = 0;
			w3.Expand = false;
			w3.Fill = false;
			// Container child vbox.Gtk.Box+BoxChild
			this.progressbar = new global::Gtk.ProgressBar ();
			this.progressbar.Name = "progressbar";
			this.vbox.Add (this.progressbar);
			global::Gtk.Box.BoxChild w4 = ((global::Gtk.Box.BoxChild)(this.vbox[this.progressbar]));
			w4.Position = 1;
			w4.Expand = false;
			w4.Fill = false;
			this.Add (this.vbox);
			if ((this.Child != null)) {
				this.Child.ShowAll ();
			}
			this.DefaultWidth = 400;
			this.DefaultHeight = 114;
			this.Show ();
		}
	}
}
