
// This file has been generated by the GUI designer. Do not modify.
namespace MonoDevelop.Database.Sql.MySql
{
	public partial class MySqlCreateDatabaseWidget
	{
		private global::Gtk.Table table;

		private global::Gtk.ComboBoxEntry comboCharset;

		private global::Gtk.ComboBoxEntry comboCollation;

		private global::Gtk.Label label2;

		private global::Gtk.Label label3;

		protected virtual void Build ()
		{
			global::Stetic.Gui.Initialize (this);
			// Widget MonoDevelop.Database.Sql.MySql.MySqlCreateDatabaseWidget
			global::Stetic.BinContainer.Attach (this);
			this.Name = "MonoDevelop.Database.Sql.MySql.MySqlCreateDatabaseWidget";
			// Container child MonoDevelop.Database.Sql.MySql.MySqlCreateDatabaseWidget.Gtk.Container+ContainerChild
			this.table = new global::Gtk.Table (((uint)(2)), ((uint)(2)), false);
			this.table.Name = "table";
			this.table.RowSpacing = ((uint)(6));
			this.table.ColumnSpacing = ((uint)(6));
			this.table.BorderWidth = ((uint)(6));
			// Container child table.Gtk.Table+TableChild
			this.comboCharset = global::Gtk.ComboBoxEntry.NewText ();
			this.comboCharset.Name = "comboCharset";
			this.table.Add (this.comboCharset);
			global::Gtk.Table.TableChild w1 = ((global::Gtk.Table.TableChild)(this.table[this.comboCharset]));
			w1.LeftAttach = ((uint)(1));
			w1.RightAttach = ((uint)(2));
			w1.XOptions = ((global::Gtk.AttachOptions)(4));
			w1.YOptions = ((global::Gtk.AttachOptions)(4));
			// Container child table.Gtk.Table+TableChild
			this.comboCollation = global::Gtk.ComboBoxEntry.NewText ();
			this.comboCollation.Name = "comboCollation";
			this.table.Add (this.comboCollation);
			global::Gtk.Table.TableChild w2 = ((global::Gtk.Table.TableChild)(this.table[this.comboCollation]));
			w2.TopAttach = ((uint)(1));
			w2.BottomAttach = ((uint)(2));
			w2.LeftAttach = ((uint)(1));
			w2.RightAttach = ((uint)(2));
			w2.XOptions = ((global::Gtk.AttachOptions)(4));
			w2.YOptions = ((global::Gtk.AttachOptions)(4));
			// Container child table.Gtk.Table+TableChild
			this.label2 = new global::Gtk.Label ();
			this.label2.Name = "label2";
			this.label2.Xalign = 0f;
			this.label2.LabelProp = global::MonoDevelop.Database.AddinCatalog.GetString ("Character Set:");
			this.table.Add (this.label2);
			global::Gtk.Table.TableChild w3 = ((global::Gtk.Table.TableChild)(this.table[this.label2]));
			w3.XOptions = ((global::Gtk.AttachOptions)(4));
			w3.YOptions = ((global::Gtk.AttachOptions)(4));
			// Container child table.Gtk.Table+TableChild
			this.label3 = new global::Gtk.Label ();
			this.label3.Name = "label3";
			this.label3.Xalign = 0f;
			this.label3.LabelProp = global::MonoDevelop.Database.AddinCatalog.GetString ("Collation:");
			this.table.Add (this.label3);
			global::Gtk.Table.TableChild w4 = ((global::Gtk.Table.TableChild)(this.table[this.label3]));
			w4.TopAttach = ((uint)(1));
			w4.BottomAttach = ((uint)(2));
			w4.XOptions = ((global::Gtk.AttachOptions)(4));
			w4.YOptions = ((global::Gtk.AttachOptions)(4));
			this.Add (this.table);
			if ((this.Child != null)) {
				this.Child.ShowAll ();
			}
			this.Show ();
		}
	}
}
