
// This file has been generated by the GUI designer. Do not modify.
namespace MonoDevelop.Database.Sql.MySql
{
	public partial class MySqlColumnEditorWidget
	{
		private global::Gtk.VBox vbox2;

		protected virtual void Build ()
		{
			global::Stetic.Gui.Initialize (this);
			// Widget MonoDevelop.Database.Sql.MySql.MySqlColumnEditorWidget
			global::Stetic.BinContainer.Attach (this);
			this.Name = "MonoDevelop.Database.Sql.MySql.MySqlColumnEditorWidget";
			// Container child MonoDevelop.Database.Sql.MySql.MySqlColumnEditorWidget.Gtk.Container+ContainerChild
			this.vbox2 = new global::Gtk.VBox ();
			this.vbox2.Name = "vbox2";
			this.vbox2.Spacing = 6;
			this.Add (this.vbox2);
			if ((this.Child != null)) {
				this.Child.ShowAll ();
			}
			this.Show ();
		}
	}
}
