
// This file has been generated by the GUI designer. Do not modify.
namespace MonoDevelop.Database.Components
{
	public partial class ShowImageDialog
	{
		private global::Gtk.ScrolledWindow scrolledwindow;

		private global::Gtk.Image image1;

		private global::Gtk.HBox hboxError;

		private global::Gtk.Image image;

		private global::Gtk.Label label;

		private global::Gtk.Button buttonClose;

		protected virtual void Build ()
		{
			global::Stetic.Gui.Initialize (this);
			// Widget MonoDevelop.Database.Components.ShowImageDialog
			this.Name = "MonoDevelop.Database.Components.ShowImageDialog";
			this.Title = global::MonoDevelop.Database.AddinCatalog.GetString ("Image");
			this.TypeHint = ((global::Gdk.WindowTypeHint)(1));
			this.WindowPosition = ((global::Gtk.WindowPosition)(1));
			this.SkipTaskbarHint = true;
			// Internal child MonoDevelop.Database.Components.ShowImageDialog.VBox
			global::Gtk.VBox w1 = this.VBox;
			w1.Name = "dialog1_VBox";
			w1.BorderWidth = ((uint)(2));
			// Container child dialog1_VBox.Gtk.Box+BoxChild
			this.scrolledwindow = new global::Gtk.ScrolledWindow ();
			this.scrolledwindow.CanFocus = true;
			this.scrolledwindow.Name = "scrolledwindow";
			this.scrolledwindow.ShadowType = ((global::Gtk.ShadowType)(1));
			// Container child scrolledwindow.Gtk.Container+ContainerChild
			global::Gtk.Viewport w2 = new global::Gtk.Viewport ();
			w2.ShadowType = ((global::Gtk.ShadowType)(0));
			// Container child GtkViewport.Gtk.Container+ContainerChild
			this.image1 = new global::Gtk.Image ();
			this.image1.Name = "image1";
			w2.Add (this.image1);
			this.scrolledwindow.Add (w2);
			w1.Add (this.scrolledwindow);
			global::Gtk.Box.BoxChild w5 = ((global::Gtk.Box.BoxChild)(w1[this.scrolledwindow]));
			w5.Position = 0;
			// Container child dialog1_VBox.Gtk.Box+BoxChild
			this.hboxError = new global::Gtk.HBox ();
			this.hboxError.Name = "hboxError";
			this.hboxError.Spacing = 6;
			// Container child hboxError.Gtk.Box+BoxChild
			this.image = new global::Gtk.Image ();
			this.image.Name = "image";
			this.image.Pixbuf = global::Stetic.IconLoader.LoadIcon (this, "gtk-dialog-error", global::Gtk.IconSize.Menu);
			this.hboxError.Add (this.image);
			global::Gtk.Box.BoxChild w6 = ((global::Gtk.Box.BoxChild)(this.hboxError[this.image]));
			w6.Position = 0;
			w6.Expand = false;
			w6.Fill = false;
			// Container child hboxError.Gtk.Box+BoxChild
			this.label = new global::Gtk.Label ();
			this.label.Name = "label";
			this.label.Xalign = 0f;
			this.label.LabelProp = global::MonoDevelop.Database.AddinCatalog.GetString ("Unable to load object as Image");
			this.hboxError.Add (this.label);
			global::Gtk.Box.BoxChild w7 = ((global::Gtk.Box.BoxChild)(this.hboxError[this.label]));
			w7.Position = 1;
			w1.Add (this.hboxError);
			global::Gtk.Box.BoxChild w8 = ((global::Gtk.Box.BoxChild)(w1[this.hboxError]));
			w8.Position = 1;
			w8.Expand = false;
			w8.Fill = false;
			// Internal child MonoDevelop.Database.Components.ShowImageDialog.ActionArea
			global::Gtk.HButtonBox w9 = this.ActionArea;
			w9.Name = "dialog1_ActionArea";
			w9.Spacing = 6;
			w9.BorderWidth = ((uint)(5));
			w9.LayoutStyle = ((global::Gtk.ButtonBoxStyle)(4));
			// Container child dialog1_ActionArea.Gtk.ButtonBox+ButtonBoxChild
			this.buttonClose = new global::Gtk.Button ();
			this.buttonClose.CanDefault = true;
			this.buttonClose.CanFocus = true;
			this.buttonClose.Name = "buttonClose";
			this.buttonClose.UseStock = true;
			this.buttonClose.UseUnderline = true;
			this.buttonClose.Label = "gtk-close";
			this.AddActionWidget (this.buttonClose, -7);
			global::Gtk.ButtonBox.ButtonBoxChild w10 = ((global::Gtk.ButtonBox.ButtonBoxChild)(w9[this.buttonClose]));
			w10.Expand = false;
			w10.Fill = false;
			if ((this.Child != null)) {
				this.Child.ShowAll ();
			}
			this.DefaultWidth = 400;
			this.DefaultHeight = 300;
			this.hboxError.Hide ();
			this.Show ();
			this.buttonClose.Clicked += new global::System.EventHandler (this.CloseClicked);
		}
	}
}
