
// This file has been generated by the GUI designer. Do not modify.
namespace MonoDevelop.Database.Designer
{
	public partial class UniqueConstraintEditorWidget
	{
		private global::Gtk.HPaned hpaned;

		private global::Gtk.VBox vbox3;

		private global::Gtk.ScrolledWindow windowPK;

		private global::Gtk.TreeView listUnique;

		private global::Gtk.HButtonBox hbuttonbox;

		private global::Gtk.Button buttonAdd;

		private global::Gtk.Button buttonRemove;

		private global::MonoDevelop.Database.Components.SelectColumnWidget columnSelecter;

		protected virtual void Build ()
		{
			global::Stetic.Gui.Initialize (this);
			// Widget MonoDevelop.Database.Designer.UniqueConstraintEditorWidget
			global::Stetic.BinContainer.Attach (this);
			this.Name = "MonoDevelop.Database.Designer.UniqueConstraintEditorWidget";
			// Container child MonoDevelop.Database.Designer.UniqueConstraintEditorWidget.Gtk.Container+ContainerChild
			this.hpaned = new global::Gtk.HPaned ();
			this.hpaned.CanFocus = true;
			this.hpaned.Name = "hpaned";
			this.hpaned.Position = 304;
			// Container child hpaned.Gtk.Paned+PanedChild
			this.vbox3 = new global::Gtk.VBox ();
			this.vbox3.Name = "vbox3";
			this.vbox3.Spacing = 6;
			// Container child vbox3.Gtk.Box+BoxChild
			this.windowPK = new global::Gtk.ScrolledWindow ();
			this.windowPK.CanFocus = true;
			this.windowPK.Name = "windowPK";
			this.windowPK.ShadowType = ((global::Gtk.ShadowType)(1));
			// Container child windowPK.Gtk.Container+ContainerChild
			this.listUnique = new global::Gtk.TreeView ();
			this.listUnique.CanFocus = true;
			this.listUnique.Name = "listUnique";
			this.windowPK.Add (this.listUnique);
			this.vbox3.Add (this.windowPK);
			global::Gtk.Box.BoxChild w2 = ((global::Gtk.Box.BoxChild)(this.vbox3[this.windowPK]));
			w2.Position = 0;
			// Container child vbox3.Gtk.Box+BoxChild
			this.hbuttonbox = new global::Gtk.HButtonBox ();
			this.hbuttonbox.Name = "hbuttonbox";
			this.hbuttonbox.Spacing = 6;
			this.hbuttonbox.LayoutStyle = ((global::Gtk.ButtonBoxStyle)(3));
			// Container child hbuttonbox.Gtk.ButtonBox+ButtonBoxChild
			this.buttonAdd = new global::Gtk.Button ();
			this.buttonAdd.CanFocus = true;
			this.buttonAdd.Name = "buttonAdd";
			this.buttonAdd.UseStock = true;
			this.buttonAdd.UseUnderline = true;
			this.buttonAdd.Label = "gtk-add";
			this.hbuttonbox.Add (this.buttonAdd);
			global::Gtk.ButtonBox.ButtonBoxChild w3 = ((global::Gtk.ButtonBox.ButtonBoxChild)(this.hbuttonbox[this.buttonAdd]));
			w3.Expand = false;
			w3.Fill = false;
			// Container child hbuttonbox.Gtk.ButtonBox+ButtonBoxChild
			this.buttonRemove = new global::Gtk.Button ();
			this.buttonRemove.Sensitive = false;
			this.buttonRemove.CanFocus = true;
			this.buttonRemove.Name = "buttonRemove";
			this.buttonRemove.UseStock = true;
			this.buttonRemove.UseUnderline = true;
			this.buttonRemove.Label = "gtk-remove";
			this.hbuttonbox.Add (this.buttonRemove);
			global::Gtk.ButtonBox.ButtonBoxChild w4 = ((global::Gtk.ButtonBox.ButtonBoxChild)(this.hbuttonbox[this.buttonRemove]));
			w4.Position = 1;
			w4.Expand = false;
			w4.Fill = false;
			this.vbox3.Add (this.hbuttonbox);
			global::Gtk.Box.BoxChild w5 = ((global::Gtk.Box.BoxChild)(this.vbox3[this.hbuttonbox]));
			w5.Position = 1;
			w5.Expand = false;
			w5.Fill = false;
			this.hpaned.Add (this.vbox3);
			global::Gtk.Paned.PanedChild w6 = ((global::Gtk.Paned.PanedChild)(this.hpaned[this.vbox3]));
			w6.Resize = false;
			// Container child hpaned.Gtk.Paned+PanedChild
			this.columnSelecter = new global::MonoDevelop.Database.Components.SelectColumnWidget ();
			this.columnSelecter.Sensitive = false;
			this.columnSelecter.CanFocus = true;
			this.columnSelecter.Name = "columnSelecter";
			this.columnSelecter.ShadowType = ((global::Gtk.ShadowType)(1));
			this.columnSelecter.SingleCheck = false;
			this.hpaned.Add (this.columnSelecter);
			this.Add (this.hpaned);
			if ((this.Child != null)) {
				this.Child.ShowAll ();
			}
			this.Show ();
			this.buttonAdd.Clicked += new global::System.EventHandler (this.AddClicked);
			this.buttonRemove.Clicked += new global::System.EventHandler (this.RemoveClicked);
		}
	}
}
