
// This file has been generated by the GUI designer. Do not modify.
namespace MonoDevelop.Database.Components
{
	public partial class DatabaseConnectionEditorWidget
	{
		private global::Gtk.HBox hbox3;

		private global::MonoDevelop.Database.Components.DatabaseConnectionContextComboBox comboConnection;

		private global::Gtk.Button buttonNew;

		private global::Gtk.Button buttonEdit;

		protected virtual void Build ()
		{
			global::Stetic.Gui.Initialize (this);
			// Widget MonoDevelop.Database.Components.DatabaseConnectionEditorWidget
			global::Stetic.BinContainer.Attach (this);
			this.Name = "MonoDevelop.Database.Components.DatabaseConnectionEditorWidget";
			// Container child MonoDevelop.Database.Components.DatabaseConnectionEditorWidget.Gtk.Container+ContainerChild
			this.hbox3 = new global::Gtk.HBox ();
			this.hbox3.Name = "hbox3";
			this.hbox3.Spacing = 6;
			// Container child hbox3.Gtk.Box+BoxChild
			this.comboConnection = new global::MonoDevelop.Database.Components.DatabaseConnectionContextComboBox ();
			this.comboConnection.Name = "comboConnection";
			this.hbox3.Add (this.comboConnection);
			global::Gtk.Box.BoxChild w1 = ((global::Gtk.Box.BoxChild)(this.hbox3[this.comboConnection]));
			w1.Position = 0;
			// Container child hbox3.Gtk.Box+BoxChild
			this.buttonNew = new global::Gtk.Button ();
			this.buttonNew.CanFocus = true;
			this.buttonNew.Name = "buttonNew";
			this.buttonNew.UseUnderline = true;
			// Container child buttonNew.Gtk.Container+ContainerChild
			global::Gtk.Alignment w2 = new global::Gtk.Alignment (0.5f, 0.5f, 0f, 0f);
			// Container child GtkAlignment.Gtk.Container+ContainerChild
			global::Gtk.HBox w3 = new global::Gtk.HBox ();
			w3.Spacing = 2;
			// Container child GtkHBox.Gtk.Container+ContainerChild
			global::Gtk.Image w4 = new global::Gtk.Image ();
			w4.Pixbuf = global::Stetic.IconLoader.LoadIcon (this, "gtk-new", global::Gtk.IconSize.Menu);
			w3.Add (w4);
			// Container child GtkHBox.Gtk.Container+ContainerChild
			global::Gtk.Label w6 = new global::Gtk.Label ();
			w6.LabelProp = global::MonoDevelop.Database.AddinCatalog.GetString ("_New");
			w6.UseUnderline = true;
			w3.Add (w6);
			w2.Add (w3);
			this.buttonNew.Add (w2);
			this.hbox3.Add (this.buttonNew);
			global::Gtk.Box.BoxChild w10 = ((global::Gtk.Box.BoxChild)(this.hbox3[this.buttonNew]));
			w10.Position = 1;
			w10.Expand = false;
			w10.Fill = false;
			// Container child hbox3.Gtk.Box+BoxChild
			this.buttonEdit = new global::Gtk.Button ();
			this.buttonEdit.CanFocus = true;
			this.buttonEdit.Name = "buttonEdit";
			this.buttonEdit.UseUnderline = true;
			// Container child buttonEdit.Gtk.Container+ContainerChild
			global::Gtk.Alignment w11 = new global::Gtk.Alignment (0.5f, 0.5f, 0f, 0f);
			// Container child GtkAlignment.Gtk.Container+ContainerChild
			global::Gtk.HBox w12 = new global::Gtk.HBox ();
			w12.Spacing = 2;
			// Container child GtkHBox.Gtk.Container+ContainerChild
			global::Gtk.Image w13 = new global::Gtk.Image ();
			w13.Pixbuf = global::Stetic.IconLoader.LoadIcon (this, "gtk-edit", global::Gtk.IconSize.Menu);
			w12.Add (w13);
			// Container child GtkHBox.Gtk.Container+ContainerChild
			global::Gtk.Label w15 = new global::Gtk.Label ();
			w15.LabelProp = global::MonoDevelop.Database.AddinCatalog.GetString ("_Edit");
			w15.UseUnderline = true;
			w12.Add (w15);
			w11.Add (w12);
			this.buttonEdit.Add (w11);
			this.hbox3.Add (this.buttonEdit);
			global::Gtk.Box.BoxChild w19 = ((global::Gtk.Box.BoxChild)(this.hbox3[this.buttonEdit]));
			w19.Position = 2;
			w19.Expand = false;
			w19.Fill = false;
			this.Add (this.hbox3);
			if ((this.Child != null)) {
				this.Child.ShowAll ();
			}
			this.Hide ();
			this.comboConnection.Changed += new global::System.EventHandler (this.OnComboConnectionChanged);
			this.buttonNew.Clicked += new global::System.EventHandler (this.OnButtonNewClicked);
			this.buttonEdit.Clicked += new global::System.EventHandler (this.OnButtonEditClicked);
		}
	}
}
