// Autogenerated from MonoDevelop.Database.Designer.addin.xml

using System.Reflection;

[assembly: AssemblyProduct ("MonoDevelop")]
[assembly: AssemblyTitle ("MonoDevelop Database Designer")]
[assembly: AssemblyDescription ("Database Designer Module")]
[assembly: AssemblyVersion ("2.6")]
[assembly: AssemblyCopyright ("MIT X11")]
