//
// Authors:
//   Ben Motmans  <ben.motmans@gmail.com>
//
// Copyright (c) 2007 Ben Motmans
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
//

using Gtk;
using System;
using System.Collections.Generic;
using MonoDevelop.Database.Sql;

namespace MonoDevelop.Database.Components
{
	public class CellRendererTypeCombo : CellRendererCombo
	{
		private ListStore store;
		
		public CellRendererTypeCombo ()
		{
			store = new ListStore (typeof (string), typeof (Type));
			
			store.AppendValues ("bool", typeof (bool));
			store.AppendValues ("byte array", typeof (byte[]));
			store.AppendValues ("char", typeof (char));
			store.AppendValues ("date/time", typeof (DateTime));
			store.AppendValues ("decimal", typeof (decimal));
			store.AppendValues ("double", typeof (double));
			//TODO: store.AppendValues ("enum", typeof (Enum));
			store.AppendValues ("short", typeof (short));
			store.AppendValues ("integer", typeof (int));
			store.AppendValues ("long", typeof (long));
			store.AppendValues ("unsigned short", typeof (ushort));
			store.AppendValues ("unsigned integer", typeof (uint));
			store.AppendValues ("unsigned long", typeof (ulong));
			store.AppendValues ("float", typeof (float));
			
			Model = store;
		}
	}
}