
// This file has been generated by the GUI designer. Do not modify.
namespace MonoDevelop.Database.Components
{
	public partial class ShowTextDialog
	{
		private global::Gtk.VBox vboxContent;

		private global::Gtk.Button button462;

		protected virtual void Build ()
		{
			global::Stetic.Gui.Initialize (this);
			// Widget MonoDevelop.Database.Components.ShowTextDialog
			this.Name = "MonoDevelop.Database.Components.ShowTextDialog";
			this.Title = global::MonoDevelop.Database.AddinCatalog.GetString ("Text");
			this.TypeHint = ((global::Gdk.WindowTypeHint)(1));
			this.WindowPosition = ((global::Gtk.WindowPosition)(1));
			this.SkipTaskbarHint = true;
			// Internal child MonoDevelop.Database.Components.ShowTextDialog.VBox
			global::Gtk.VBox w1 = this.VBox;
			w1.Name = "vbox";
			w1.BorderWidth = ((uint)(2));
			// Container child vbox.Gtk.Box+BoxChild
			this.vboxContent = new global::Gtk.VBox ();
			this.vboxContent.Name = "vboxContent";
			this.vboxContent.Spacing = 6;
			this.vboxContent.BorderWidth = ((uint)(6));
			w1.Add (this.vboxContent);
			global::Gtk.Box.BoxChild w2 = ((global::Gtk.Box.BoxChild)(w1[this.vboxContent]));
			w2.Position = 0;
			// Internal child MonoDevelop.Database.Components.ShowTextDialog.ActionArea
			global::Gtk.HButtonBox w3 = this.ActionArea;
			w3.Name = "GtkDialog_ActionArea";
			w3.LayoutStyle = ((global::Gtk.ButtonBoxStyle)(4));
			// Container child GtkDialog_ActionArea.Gtk.ButtonBox+ButtonBoxChild
			this.button462 = new global::Gtk.Button ();
			this.button462.CanFocus = true;
			this.button462.Name = "button462";
			this.button462.UseUnderline = true;
			this.button462.Label = global::MonoDevelop.Database.AddinCatalog.GetString ("button462");
			this.AddActionWidget (this.button462, 0);
			global::Gtk.ButtonBox.ButtonBoxChild w4 = ((global::Gtk.ButtonBox.ButtonBoxChild)(w3[this.button462]));
			w4.Expand = false;
			w4.Fill = false;
			if ((this.Child != null)) {
				this.Child.ShowAll ();
			}
			this.DefaultWidth = 400;
			this.DefaultHeight = 300;
			this.Show ();
		}
	}
}
