
// This file has been generated by the GUI designer. Do not modify.
namespace MonoDevelop.Database.CodeGenerator
{
	public partial class GenerateDataClass
	{
		private global::Gtk.HBox hbox1;

		private global::Gtk.Label label1;

		private global::MonoDevelop.Database.Components.DatabaseConnectionEditorWidget comboProviderConnection;

		private global::Gtk.Label labelMessage;

		private global::Gtk.HSeparator hseparator1;

		private global::Gtk.HBox hbox2;

		private global::Gtk.Label label2;

		private global::Gtk.ComboBox comboOutput;

		private global::Gtk.Label label3;

		private global::Gtk.ComboBox comboLanguage;

		private global::Gtk.Label label4;

		private global::Gtk.ComboBox comboStyle;

		private global::Gtk.Entry entryFile;

		private global::Gtk.HSeparator hseparator2;

		private global::Gtk.Table table1;

		private global::MonoDevelop.Database.Components.ProjectDirectoryComboBox comboProject;

		private global::Gtk.Entry entryNamespace;

		private global::Gtk.Label label5;

		private global::Gtk.Label label6;

		private global::Gtk.HSeparator hseparator3;

		private global::Gtk.Table table2;

		private global::Gtk.Entry entryEntityAttr;

		private global::Gtk.Entry entryEntityBase;

		private global::Gtk.Entry entryGenerateType;

		private global::Gtk.Entry entryMemberAttr;

		private global::Gtk.Label label10;

		private global::Gtk.Label label11;

		private global::Gtk.Label label8;

		private global::Gtk.Label label9;

		private global::Gtk.HSeparator hseparator4;

		private global::Gtk.HBox hbox4;

		private global::Gtk.Label label12;

		private global::Gtk.Entry entryCulture;

		private global::Gtk.Expander expander1;

		private global::Gtk.ScrolledWindow scrolledwindow1;

		private global::Gtk.VBox vbox2;

		private global::Gtk.CheckButton checkSchema;

		private global::Gtk.CheckButton checkGenerateTimestamps;

		private global::Gtk.CheckButton checkEqualsAndHash;

		private global::Gtk.CheckButton checkSprocs;

		private global::Gtk.CheckButton checkPluralize;

		private global::Gtk.Label GtkLabel4;

		private global::Gtk.Button buttonCancel;

		private global::Gtk.Button buttonOk;

		protected virtual void Build ()
		{
			global::Stetic.Gui.Initialize (this);
			// Widget MonoDevelop.Database.CodeGenerator.GenerateDataClass
			this.Name = "MonoDevelop.Database.CodeGenerator.GenerateDataClass";
			this.Title = global::Mono.Unix.Catalog.GetString ("Generate Linq Class");
			this.WindowPosition = ((global::Gtk.WindowPosition)(4));
			// Internal child MonoDevelop.Database.CodeGenerator.GenerateDataClass.VBox
			global::Gtk.VBox w1 = this.VBox;
			w1.Name = "dialog1_VBox";
			w1.BorderWidth = ((uint)(2));
			// Container child dialog1_VBox.Gtk.Box+BoxChild
			this.hbox1 = new global::Gtk.HBox ();
			this.hbox1.Name = "hbox1";
			this.hbox1.Spacing = 6;
			// Container child hbox1.Gtk.Box+BoxChild
			this.label1 = new global::Gtk.Label ();
			this.label1.Name = "label1";
			this.label1.LabelProp = global::Mono.Unix.Catalog.GetString ("Connection");
			this.hbox1.Add (this.label1);
			global::Gtk.Box.BoxChild w2 = ((global::Gtk.Box.BoxChild)(this.hbox1[this.label1]));
			w2.Position = 0;
			w2.Expand = false;
			w2.Fill = false;
			// Container child hbox1.Gtk.Box+BoxChild
			this.comboProviderConnection = new global::MonoDevelop.Database.Components.DatabaseConnectionEditorWidget ();
			this.comboProviderConnection.Events = ((global::Gdk.EventMask)(256));
			this.comboProviderConnection.Name = "comboProviderConnection";
			this.hbox1.Add (this.comboProviderConnection);
			global::Gtk.Box.BoxChild w3 = ((global::Gtk.Box.BoxChild)(this.hbox1[this.comboProviderConnection]));
			w3.Position = 1;
			w1.Add (this.hbox1);
			global::Gtk.Box.BoxChild w4 = ((global::Gtk.Box.BoxChild)(w1[this.hbox1]));
			w4.Position = 0;
			w4.Expand = false;
			w4.Fill = false;
			// Container child dialog1_VBox.Gtk.Box+BoxChild
			this.labelMessage = new global::Gtk.Label ();
			this.labelMessage.Name = "labelMessage";
			w1.Add (this.labelMessage);
			global::Gtk.Box.BoxChild w5 = ((global::Gtk.Box.BoxChild)(w1[this.labelMessage]));
			w5.Position = 1;
			w5.Expand = false;
			w5.Fill = false;
			// Container child dialog1_VBox.Gtk.Box+BoxChild
			this.hseparator1 = new global::Gtk.HSeparator ();
			this.hseparator1.Name = "hseparator1";
			w1.Add (this.hseparator1);
			global::Gtk.Box.BoxChild w6 = ((global::Gtk.Box.BoxChild)(w1[this.hseparator1]));
			w6.Position = 2;
			w6.Expand = false;
			w6.Fill = false;
			w6.Padding = ((uint)(5));
			// Container child dialog1_VBox.Gtk.Box+BoxChild
			this.hbox2 = new global::Gtk.HBox ();
			this.hbox2.Name = "hbox2";
			this.hbox2.Spacing = 6;
			// Container child hbox2.Gtk.Box+BoxChild
			this.label2 = new global::Gtk.Label ();
			this.label2.Name = "label2";
			this.label2.LabelProp = global::Mono.Unix.Catalog.GetString ("Output");
			this.hbox2.Add (this.label2);
			global::Gtk.Box.BoxChild w7 = ((global::Gtk.Box.BoxChild)(this.hbox2[this.label2]));
			w7.Position = 0;
			w7.Expand = false;
			w7.Fill = false;
			// Container child hbox2.Gtk.Box+BoxChild
			this.comboOutput = global::Gtk.ComboBox.NewText ();
			this.comboOutput.AppendText (global::Mono.Unix.Catalog.GetString ("Code"));
			this.comboOutput.AppendText (global::Mono.Unix.Catalog.GetString ("DBML"));
			this.comboOutput.AppendText (global::Mono.Unix.Catalog.GetString ("Code & DBML"));
			this.comboOutput.Name = "comboOutput";
			this.comboOutput.Active = 0;
			this.hbox2.Add (this.comboOutput);
			global::Gtk.Box.BoxChild w8 = ((global::Gtk.Box.BoxChild)(this.hbox2[this.comboOutput]));
			w8.Position = 1;
			w8.Expand = false;
			w8.Fill = false;
			// Container child hbox2.Gtk.Box+BoxChild
			this.label3 = new global::Gtk.Label ();
			this.label3.Name = "label3";
			this.label3.LabelProp = global::Mono.Unix.Catalog.GetString ("Language");
			this.hbox2.Add (this.label3);
			global::Gtk.Box.BoxChild w9 = ((global::Gtk.Box.BoxChild)(this.hbox2[this.label3]));
			w9.Position = 2;
			w9.Expand = false;
			w9.Fill = false;
			// Container child hbox2.Gtk.Box+BoxChild
			this.comboLanguage = global::Gtk.ComboBox.NewText ();
			this.comboLanguage.AppendText (global::Mono.Unix.Catalog.GetString ("C#2"));
			this.comboLanguage.AppendText (global::Mono.Unix.Catalog.GetString ("C#"));
			this.comboLanguage.AppendText (global::Mono.Unix.Catalog.GetString ("VB"));
			this.comboLanguage.Name = "comboLanguage";
			this.comboLanguage.Active = 0;
			this.hbox2.Add (this.comboLanguage);
			global::Gtk.Box.BoxChild w10 = ((global::Gtk.Box.BoxChild)(this.hbox2[this.comboLanguage]));
			w10.Position = 3;
			w10.Expand = false;
			w10.Fill = false;
			// Container child hbox2.Gtk.Box+BoxChild
			this.label4 = new global::Gtk.Label ();
			this.label4.Name = "label4";
			this.label4.LabelProp = global::Mono.Unix.Catalog.GetString ("Style");
			this.hbox2.Add (this.label4);
			global::Gtk.Box.BoxChild w11 = ((global::Gtk.Box.BoxChild)(this.hbox2[this.label4]));
			w11.Position = 4;
			w11.Expand = false;
			w11.Fill = false;
			// Container child hbox2.Gtk.Box+BoxChild
			this.comboStyle = global::Gtk.ComboBox.NewText ();
			this.comboStyle.AppendText (global::Mono.Unix.Catalog.GetString ("NET"));
			this.comboStyle.AppendText (global::Mono.Unix.Catalog.GetString ("Leave"));
			this.comboStyle.AppendText (global::Mono.Unix.Catalog.GetString ("Pascal"));
			this.comboStyle.AppendText (global::Mono.Unix.Catalog.GetString ("Camel"));
			this.comboStyle.Name = "comboStyle";
			this.comboStyle.Active = 0;
			this.hbox2.Add (this.comboStyle);
			global::Gtk.Box.BoxChild w12 = ((global::Gtk.Box.BoxChild)(this.hbox2[this.comboStyle]));
			w12.Position = 5;
			w12.Expand = false;
			w12.Fill = false;
			// Container child hbox2.Gtk.Box+BoxChild
			this.entryFile = new global::Gtk.Entry ();
			this.entryFile.CanFocus = true;
			this.entryFile.Name = "entryFile";
			this.entryFile.Text = global::Mono.Unix.Catalog.GetString ("Output.cs");
			this.entryFile.IsEditable = true;
			this.entryFile.InvisibleChar = '●';
			this.hbox2.Add (this.entryFile);
			global::Gtk.Box.BoxChild w13 = ((global::Gtk.Box.BoxChild)(this.hbox2[this.entryFile]));
			w13.Position = 6;
			w1.Add (this.hbox2);
			global::Gtk.Box.BoxChild w14 = ((global::Gtk.Box.BoxChild)(w1[this.hbox2]));
			w14.Position = 3;
			w14.Expand = false;
			w14.Fill = false;
			// Container child dialog1_VBox.Gtk.Box+BoxChild
			this.hseparator2 = new global::Gtk.HSeparator ();
			this.hseparator2.Name = "hseparator2";
			w1.Add (this.hseparator2);
			global::Gtk.Box.BoxChild w15 = ((global::Gtk.Box.BoxChild)(w1[this.hseparator2]));
			w15.Position = 4;
			w15.Expand = false;
			w15.Fill = false;
			w15.Padding = ((uint)(5));
			// Container child dialog1_VBox.Gtk.Box+BoxChild
			this.table1 = new global::Gtk.Table (((uint)(2)), ((uint)(2)), false);
			this.table1.Name = "table1";
			this.table1.RowSpacing = ((uint)(6));
			this.table1.ColumnSpacing = ((uint)(6));
			// Container child table1.Gtk.Table+TableChild
			this.comboProject = new global::MonoDevelop.Database.Components.ProjectDirectoryComboBox ();
			this.comboProject.Name = "comboProject";
			this.table1.Add (this.comboProject);
			global::Gtk.Table.TableChild w16 = ((global::Gtk.Table.TableChild)(this.table1[this.comboProject]));
			w16.LeftAttach = ((uint)(1));
			w16.RightAttach = ((uint)(2));
			w16.YOptions = ((global::Gtk.AttachOptions)(4));
			// Container child table1.Gtk.Table+TableChild
			this.entryNamespace = new global::Gtk.Entry ();
			this.entryNamespace.CanFocus = true;
			this.entryNamespace.Name = "entryNamespace";
			this.entryNamespace.IsEditable = true;
			this.entryNamespace.InvisibleChar = '●';
			this.table1.Add (this.entryNamespace);
			global::Gtk.Table.TableChild w17 = ((global::Gtk.Table.TableChild)(this.table1[this.entryNamespace]));
			w17.TopAttach = ((uint)(1));
			w17.BottomAttach = ((uint)(2));
			w17.LeftAttach = ((uint)(1));
			w17.RightAttach = ((uint)(2));
			w17.XOptions = ((global::Gtk.AttachOptions)(4));
			w17.YOptions = ((global::Gtk.AttachOptions)(4));
			// Container child table1.Gtk.Table+TableChild
			this.label5 = new global::Gtk.Label ();
			this.label5.Name = "label5";
			this.label5.Xalign = 0f;
			this.label5.LabelProp = global::Mono.Unix.Catalog.GetString ("Project");
			this.table1.Add (this.label5);
			global::Gtk.Table.TableChild w18 = ((global::Gtk.Table.TableChild)(this.table1[this.label5]));
			w18.XOptions = ((global::Gtk.AttachOptions)(4));
			w18.YOptions = ((global::Gtk.AttachOptions)(4));
			// Container child table1.Gtk.Table+TableChild
			this.label6 = new global::Gtk.Label ();
			this.label6.Name = "label6";
			this.label6.Xalign = 0f;
			this.label6.LabelProp = global::Mono.Unix.Catalog.GetString ("Namespace");
			this.table1.Add (this.label6);
			global::Gtk.Table.TableChild w19 = ((global::Gtk.Table.TableChild)(this.table1[this.label6]));
			w19.TopAttach = ((uint)(1));
			w19.BottomAttach = ((uint)(2));
			w19.XOptions = ((global::Gtk.AttachOptions)(4));
			w19.YOptions = ((global::Gtk.AttachOptions)(4));
			w1.Add (this.table1);
			global::Gtk.Box.BoxChild w20 = ((global::Gtk.Box.BoxChild)(w1[this.table1]));
			w20.Position = 5;
			w20.Expand = false;
			w20.Fill = false;
			// Container child dialog1_VBox.Gtk.Box+BoxChild
			this.hseparator3 = new global::Gtk.HSeparator ();
			this.hseparator3.Name = "hseparator3";
			w1.Add (this.hseparator3);
			global::Gtk.Box.BoxChild w21 = ((global::Gtk.Box.BoxChild)(w1[this.hseparator3]));
			w21.Position = 6;
			w21.Expand = false;
			w21.Fill = false;
			w21.Padding = ((uint)(5));
			// Container child dialog1_VBox.Gtk.Box+BoxChild
			this.table2 = new global::Gtk.Table (((uint)(4)), ((uint)(2)), false);
			this.table2.Name = "table2";
			this.table2.RowSpacing = ((uint)(6));
			this.table2.ColumnSpacing = ((uint)(6));
			// Container child table2.Gtk.Table+TableChild
			this.entryEntityAttr = new global::Gtk.Entry ();
			this.entryEntityAttr.TooltipMarkup = "Comma separated ATTRIBUTE(S).";
			this.entryEntityAttr.CanFocus = true;
			this.entryEntityAttr.Name = "entryEntityAttr";
			this.entryEntityAttr.IsEditable = true;
			this.entryEntityAttr.InvisibleChar = '●';
			this.table2.Add (this.entryEntityAttr);
			global::Gtk.Table.TableChild w22 = ((global::Gtk.Table.TableChild)(this.table2[this.entryEntityAttr]));
			w22.TopAttach = ((uint)(1));
			w22.BottomAttach = ((uint)(2));
			w22.LeftAttach = ((uint)(1));
			w22.RightAttach = ((uint)(2));
			w22.YOptions = ((global::Gtk.AttachOptions)(4));
			// Container child table2.Gtk.Table+TableChild
			this.entryEntityBase = new global::Gtk.Entry ();
			this.entryEntityBase.CanFocus = true;
			this.entryEntityBase.Name = "entryEntityBase";
			this.entryEntityBase.IsEditable = true;
			this.entryEntityBase.InvisibleChar = '●';
			this.table2.Add (this.entryEntityBase);
			global::Gtk.Table.TableChild w23 = ((global::Gtk.Table.TableChild)(this.table2[this.entryEntityBase]));
			w23.LeftAttach = ((uint)(1));
			w23.RightAttach = ((uint)(2));
			w23.YOptions = ((global::Gtk.AttachOptions)(4));
			// Container child table2.Gtk.Table+TableChild
			this.entryGenerateType = new global::Gtk.Entry ();
			this.entryGenerateType.CanFocus = true;
			this.entryGenerateType.Name = "entryGenerateType";
			this.entryGenerateType.IsEditable = true;
			this.entryGenerateType.InvisibleChar = '●';
			this.table2.Add (this.entryGenerateType);
			global::Gtk.Table.TableChild w24 = ((global::Gtk.Table.TableChild)(this.table2[this.entryGenerateType]));
			w24.TopAttach = ((uint)(3));
			w24.BottomAttach = ((uint)(4));
			w24.LeftAttach = ((uint)(1));
			w24.RightAttach = ((uint)(2));
			w24.YOptions = ((global::Gtk.AttachOptions)(4));
			// Container child table2.Gtk.Table+TableChild
			this.entryMemberAttr = new global::Gtk.Entry ();
			this.entryMemberAttr.TooltipMarkup = "Comma separated ATTRIBUTE(S).";
			this.entryMemberAttr.CanFocus = true;
			this.entryMemberAttr.Name = "entryMemberAttr";
			this.entryMemberAttr.IsEditable = true;
			this.entryMemberAttr.InvisibleChar = '●';
			this.table2.Add (this.entryMemberAttr);
			global::Gtk.Table.TableChild w25 = ((global::Gtk.Table.TableChild)(this.table2[this.entryMemberAttr]));
			w25.TopAttach = ((uint)(2));
			w25.BottomAttach = ((uint)(3));
			w25.LeftAttach = ((uint)(1));
			w25.RightAttach = ((uint)(2));
			w25.YOptions = ((global::Gtk.AttachOptions)(4));
			// Container child table2.Gtk.Table+TableChild
			this.label10 = new global::Gtk.Label ();
			this.label10.Name = "label10";
			this.label10.Xalign = 0f;
			this.label10.LabelProp = global::Mono.Unix.Catalog.GetString ("Member Attributes");
			this.table2.Add (this.label10);
			global::Gtk.Table.TableChild w26 = ((global::Gtk.Table.TableChild)(this.table2[this.label10]));
			w26.TopAttach = ((uint)(2));
			w26.BottomAttach = ((uint)(3));
			w26.XOptions = ((global::Gtk.AttachOptions)(4));
			w26.YOptions = ((global::Gtk.AttachOptions)(4));
			// Container child table2.Gtk.Table+TableChild
			this.label11 = new global::Gtk.Label ();
			this.label11.Name = "label11";
			this.label11.Xalign = 0f;
			this.label11.LabelProp = global::Mono.Unix.Catalog.GetString ("Generate Type");
			this.table2.Add (this.label11);
			global::Gtk.Table.TableChild w27 = ((global::Gtk.Table.TableChild)(this.table2[this.label11]));
			w27.TopAttach = ((uint)(3));
			w27.BottomAttach = ((uint)(4));
			w27.XOptions = ((global::Gtk.AttachOptions)(4));
			w27.YOptions = ((global::Gtk.AttachOptions)(4));
			// Container child table2.Gtk.Table+TableChild
			this.label8 = new global::Gtk.Label ();
			this.label8.Name = "label8";
			this.label8.Xalign = 0f;
			this.label8.LabelProp = global::Mono.Unix.Catalog.GetString ("Entity Base");
			this.table2.Add (this.label8);
			global::Gtk.Table.TableChild w28 = ((global::Gtk.Table.TableChild)(this.table2[this.label8]));
			w28.XOptions = ((global::Gtk.AttachOptions)(4));
			w28.YOptions = ((global::Gtk.AttachOptions)(4));
			// Container child table2.Gtk.Table+TableChild
			this.label9 = new global::Gtk.Label ();
			this.label9.Name = "label9";
			this.label9.Xalign = 0f;
			this.label9.LabelProp = global::Mono.Unix.Catalog.GetString ("Entity Attributes");
			this.table2.Add (this.label9);
			global::Gtk.Table.TableChild w29 = ((global::Gtk.Table.TableChild)(this.table2[this.label9]));
			w29.TopAttach = ((uint)(1));
			w29.BottomAttach = ((uint)(2));
			w29.XOptions = ((global::Gtk.AttachOptions)(4));
			w29.YOptions = ((global::Gtk.AttachOptions)(4));
			w1.Add (this.table2);
			global::Gtk.Box.BoxChild w30 = ((global::Gtk.Box.BoxChild)(w1[this.table2]));
			w30.Position = 7;
			w30.Expand = false;
			w30.Fill = false;
			// Container child dialog1_VBox.Gtk.Box+BoxChild
			this.hseparator4 = new global::Gtk.HSeparator ();
			this.hseparator4.Name = "hseparator4";
			w1.Add (this.hseparator4);
			global::Gtk.Box.BoxChild w31 = ((global::Gtk.Box.BoxChild)(w1[this.hseparator4]));
			w31.Position = 8;
			w31.Expand = false;
			w31.Fill = false;
			w31.Padding = ((uint)(5));
			// Container child dialog1_VBox.Gtk.Box+BoxChild
			this.hbox4 = new global::Gtk.HBox ();
			this.hbox4.Name = "hbox4";
			this.hbox4.Spacing = 6;
			// Container child hbox4.Gtk.Box+BoxChild
			this.label12 = new global::Gtk.Label ();
			this.label12.Name = "label12";
			this.label12.Xalign = 0f;
			this.label12.LabelProp = global::Mono.Unix.Catalog.GetString ("Culture");
			this.hbox4.Add (this.label12);
			global::Gtk.Box.BoxChild w32 = ((global::Gtk.Box.BoxChild)(this.hbox4[this.label12]));
			w32.Position = 0;
			w32.Expand = false;
			w32.Fill = false;
			// Container child hbox4.Gtk.Box+BoxChild
			this.entryCulture = new global::Gtk.Entry ();
			this.entryCulture.WidthRequest = 50;
			this.entryCulture.CanFocus = true;
			this.entryCulture.Name = "entryCulture";
			this.entryCulture.Text = global::Mono.Unix.Catalog.GetString ("en");
			this.entryCulture.IsEditable = true;
			this.entryCulture.InvisibleChar = '●';
			this.hbox4.Add (this.entryCulture);
			global::Gtk.Box.BoxChild w33 = ((global::Gtk.Box.BoxChild)(this.hbox4[this.entryCulture]));
			w33.Position = 1;
			w33.Expand = false;
			w33.Fill = false;
			w1.Add (this.hbox4);
			global::Gtk.Box.BoxChild w34 = ((global::Gtk.Box.BoxChild)(w1[this.hbox4]));
			w34.Position = 9;
			w34.Expand = false;
			w34.Fill = false;
			// Container child dialog1_VBox.Gtk.Box+BoxChild
			this.expander1 = new global::Gtk.Expander (null);
			this.expander1.CanFocus = true;
			this.expander1.Name = "expander1";
			// Container child expander1.Gtk.Container+ContainerChild
			this.scrolledwindow1 = new global::Gtk.ScrolledWindow ();
			this.scrolledwindow1.CanFocus = true;
			this.scrolledwindow1.Name = "scrolledwindow1";
			this.scrolledwindow1.ShadowType = ((global::Gtk.ShadowType)(1));
			// Container child scrolledwindow1.Gtk.Container+ContainerChild
			global::Gtk.Viewport w35 = new global::Gtk.Viewport ();
			w35.ShadowType = ((global::Gtk.ShadowType)(0));
			// Container child GtkViewport.Gtk.Container+ContainerChild
			this.vbox2 = new global::Gtk.VBox ();
			this.vbox2.Name = "vbox2";
			this.vbox2.Spacing = 6;
			// Container child vbox2.Gtk.Box+BoxChild
			this.checkSchema = new global::Gtk.CheckButton ();
			this.checkSchema.CanFocus = true;
			this.checkSchema.Name = "checkSchema";
			this.checkSchema.Label = global::Mono.Unix.Catalog.GetString ("Generate Schema in code files.");
			this.checkSchema.Active = true;
			this.checkSchema.DrawIndicator = true;
			this.checkSchema.UseUnderline = true;
			this.vbox2.Add (this.checkSchema);
			global::Gtk.Box.BoxChild w36 = ((global::Gtk.Box.BoxChild)(this.vbox2[this.checkSchema]));
			w36.Position = 0;
			w36.Expand = false;
			w36.Fill = false;
			// Container child vbox2.Gtk.Box+BoxChild
			this.checkGenerateTimestamps = new global::Gtk.CheckButton ();
			this.checkGenerateTimestamps.CanFocus = true;
			this.checkGenerateTimestamps.Name = "checkGenerateTimestamps";
			this.checkGenerateTimestamps.Label = global::Mono.Unix.Catalog.GetString ("Generate timestamps in the generated code. ");
			this.checkGenerateTimestamps.Active = true;
			this.checkGenerateTimestamps.DrawIndicator = true;
			this.checkGenerateTimestamps.UseUnderline = true;
			this.vbox2.Add (this.checkGenerateTimestamps);
			global::Gtk.Box.BoxChild w37 = ((global::Gtk.Box.BoxChild)(this.vbox2[this.checkGenerateTimestamps]));
			w37.Position = 1;
			w37.Expand = false;
			w37.Fill = false;
			// Container child vbox2.Gtk.Box+BoxChild
			this.checkEqualsAndHash = new global::Gtk.CheckButton ();
			this.checkEqualsAndHash.CanFocus = true;
			this.checkEqualsAndHash.Name = "checkEqualsAndHash";
			this.checkEqualsAndHash.Label = global::Mono.Unix.Catalog.GetString ("Generates overrides for Equals() and GetHashCode() methods.");
			this.checkEqualsAndHash.DrawIndicator = true;
			this.checkEqualsAndHash.UseUnderline = true;
			this.vbox2.Add (this.checkEqualsAndHash);
			global::Gtk.Box.BoxChild w38 = ((global::Gtk.Box.BoxChild)(this.vbox2[this.checkEqualsAndHash]));
			w38.Position = 2;
			w38.Expand = false;
			w38.Fill = false;
			// Container child vbox2.Gtk.Box+BoxChild
			this.checkSprocs = new global::Gtk.CheckButton ();
			this.checkSprocs.CanFocus = true;
			this.checkSprocs.Name = "checkSprocs";
			this.checkSprocs.Label = global::Mono.Unix.Catalog.GetString ("Extract stored procedures.");
			this.checkSprocs.DrawIndicator = true;
			this.checkSprocs.UseUnderline = true;
			this.vbox2.Add (this.checkSprocs);
			global::Gtk.Box.BoxChild w39 = ((global::Gtk.Box.BoxChild)(this.vbox2[this.checkSprocs]));
			w39.Position = 3;
			w39.Expand = false;
			w39.Fill = false;
			// Container child vbox2.Gtk.Box+BoxChild
			this.checkPluralize = new global::Gtk.CheckButton ();
			this.checkPluralize.CanFocus = true;
			this.checkPluralize.Name = "checkPluralize";
			this.checkPluralize.Label = global::Mono.Unix.Catalog.GetString ("Automatically pluralize or singularize class and member names using specified culture rules.");
			this.checkPluralize.DrawIndicator = true;
			this.checkPluralize.UseUnderline = true;
			this.vbox2.Add (this.checkPluralize);
			global::Gtk.Box.BoxChild w40 = ((global::Gtk.Box.BoxChild)(this.vbox2[this.checkPluralize]));
			w40.Position = 4;
			w40.Expand = false;
			w40.Fill = false;
			w35.Add (this.vbox2);
			this.scrolledwindow1.Add (w35);
			this.expander1.Add (this.scrolledwindow1);
			this.GtkLabel4 = new global::Gtk.Label ();
			this.GtkLabel4.Name = "GtkLabel4";
			this.GtkLabel4.LabelProp = global::Mono.Unix.Catalog.GetString ("Advanced Options");
			this.GtkLabel4.UseUnderline = true;
			this.expander1.LabelWidget = this.GtkLabel4;
			w1.Add (this.expander1);
			global::Gtk.Box.BoxChild w44 = ((global::Gtk.Box.BoxChild)(w1[this.expander1]));
			w44.Position = 10;
			// Internal child MonoDevelop.Database.CodeGenerator.GenerateDataClass.ActionArea
			global::Gtk.HButtonBox w45 = this.ActionArea;
			w45.Name = "GenerateDataClass";
			w45.Spacing = 10;
			w45.BorderWidth = ((uint)(5));
			w45.LayoutStyle = ((global::Gtk.ButtonBoxStyle)(4));
			// Container child GenerateDataClass.Gtk.ButtonBox+ButtonBoxChild
			this.buttonCancel = new global::Gtk.Button ();
			this.buttonCancel.CanDefault = true;
			this.buttonCancel.CanFocus = true;
			this.buttonCancel.Name = "buttonCancel";
			this.buttonCancel.UseStock = true;
			this.buttonCancel.UseUnderline = true;
			this.buttonCancel.Label = "gtk-cancel";
			this.AddActionWidget (this.buttonCancel, -6);
			global::Gtk.ButtonBox.ButtonBoxChild w46 = ((global::Gtk.ButtonBox.ButtonBoxChild)(w45[this.buttonCancel]));
			w46.Expand = false;
			w46.Fill = false;
			// Container child GenerateDataClass.Gtk.ButtonBox+ButtonBoxChild
			this.buttonOk = new global::Gtk.Button ();
			this.buttonOk.Sensitive = false;
			this.buttonOk.CanDefault = true;
			this.buttonOk.CanFocus = true;
			this.buttonOk.Name = "buttonOk";
			this.buttonOk.UseStock = true;
			this.buttonOk.UseUnderline = true;
			this.buttonOk.Label = "gtk-ok";
			this.AddActionWidget (this.buttonOk, -5);
			global::Gtk.ButtonBox.ButtonBoxChild w47 = ((global::Gtk.ButtonBox.ButtonBoxChild)(w45[this.buttonOk]));
			w47.Position = 1;
			w47.Expand = false;
			w47.Fill = false;
			if ((this.Child != null)) {
				this.Child.ShowAll ();
			}
			this.DefaultWidth = 656;
			this.DefaultHeight = 436;
			this.Show ();
			this.comboProviderConnection.SelectedDatabaseChanged += new global::System.EventHandler (this.OnComboProviderConnectionSelectedDatabaseChanged);
			this.comboOutput.Changed += new global::System.EventHandler (this.OnComboOutputChanged);
			this.comboLanguage.Changed += new global::System.EventHandler (this.OnComboLanguageChanged);
			this.comboProject.Changed += new global::System.EventHandler (this.OnComboProjectChanged);
		}
	}
}
