
// This file has been generated by the GUI designer. Do not modify.
namespace MonoDevelop.Database.Designer
{
	public partial class ColumnsEditorWidget
	{
		private global::Gtk.HPaned hpaned;

		private global::Gtk.VBox vboxLeft;

		private global::Gtk.ScrolledWindow scrolledwindowLeft;

		private global::Gtk.TreeView treeColumns;

		private global::Gtk.HButtonBox hbuttonbox;

		private global::Gtk.Button buttonAdd;

		private global::Gtk.Button buttonRemove;

		private global::Gtk.Button buttonUp;

		private global::Gtk.Button buttonDown;

		private global::Gtk.ScrolledWindow scrolledwindowRight;

		protected virtual void Build ()
		{
			global::Stetic.Gui.Initialize (this);
			// Widget MonoDevelop.Database.Designer.ColumnsEditorWidget
			global::Stetic.BinContainer.Attach (this);
			this.Name = "MonoDevelop.Database.Designer.ColumnsEditorWidget";
			// Container child MonoDevelop.Database.Designer.ColumnsEditorWidget.Gtk.Container+ContainerChild
			this.hpaned = new global::Gtk.HPaned ();
			this.hpaned.CanFocus = true;
			this.hpaned.Name = "hpaned";
			this.hpaned.Position = 662;
			// Container child hpaned.Gtk.Paned+PanedChild
			this.vboxLeft = new global::Gtk.VBox ();
			this.vboxLeft.Name = "vboxLeft";
			this.vboxLeft.Spacing = 6;
			// Container child vboxLeft.Gtk.Box+BoxChild
			this.scrolledwindowLeft = new global::Gtk.ScrolledWindow ();
			this.scrolledwindowLeft.CanFocus = true;
			this.scrolledwindowLeft.Name = "scrolledwindowLeft";
			this.scrolledwindowLeft.ShadowType = ((global::Gtk.ShadowType)(1));
			// Container child scrolledwindowLeft.Gtk.Container+ContainerChild
			this.treeColumns = new global::Gtk.TreeView ();
			this.treeColumns.CanFocus = true;
			this.treeColumns.Name = "treeColumns";
			this.scrolledwindowLeft.Add (this.treeColumns);
			this.vboxLeft.Add (this.scrolledwindowLeft);
			global::Gtk.Box.BoxChild w2 = ((global::Gtk.Box.BoxChild)(this.vboxLeft[this.scrolledwindowLeft]));
			w2.Position = 0;
			// Container child vboxLeft.Gtk.Box+BoxChild
			this.hbuttonbox = new global::Gtk.HButtonBox ();
			this.hbuttonbox.Name = "hbuttonbox";
			this.hbuttonbox.Spacing = 6;
			this.hbuttonbox.LayoutStyle = ((global::Gtk.ButtonBoxStyle)(3));
			// Container child hbuttonbox.Gtk.ButtonBox+ButtonBoxChild
			this.buttonAdd = new global::Gtk.Button ();
			this.buttonAdd.CanFocus = true;
			this.buttonAdd.Name = "buttonAdd";
			this.buttonAdd.UseStock = true;
			this.buttonAdd.UseUnderline = true;
			this.buttonAdd.Label = "gtk-add";
			this.hbuttonbox.Add (this.buttonAdd);
			global::Gtk.ButtonBox.ButtonBoxChild w3 = ((global::Gtk.ButtonBox.ButtonBoxChild)(this.hbuttonbox[this.buttonAdd]));
			w3.Expand = false;
			w3.Fill = false;
			// Container child hbuttonbox.Gtk.ButtonBox+ButtonBoxChild
			this.buttonRemove = new global::Gtk.Button ();
			this.buttonRemove.Sensitive = false;
			this.buttonRemove.CanFocus = true;
			this.buttonRemove.Name = "buttonRemove";
			this.buttonRemove.UseStock = true;
			this.buttonRemove.UseUnderline = true;
			this.buttonRemove.Label = "gtk-remove";
			this.hbuttonbox.Add (this.buttonRemove);
			global::Gtk.ButtonBox.ButtonBoxChild w4 = ((global::Gtk.ButtonBox.ButtonBoxChild)(this.hbuttonbox[this.buttonRemove]));
			w4.Position = 1;
			w4.Expand = false;
			w4.Fill = false;
			// Container child hbuttonbox.Gtk.ButtonBox+ButtonBoxChild
			this.buttonUp = new global::Gtk.Button ();
			this.buttonUp.Sensitive = false;
			this.buttonUp.CanFocus = true;
			this.buttonUp.Name = "buttonUp";
			this.buttonUp.UseStock = true;
			this.buttonUp.UseUnderline = true;
			this.buttonUp.Label = "gtk-go-up";
			this.hbuttonbox.Add (this.buttonUp);
			global::Gtk.ButtonBox.ButtonBoxChild w5 = ((global::Gtk.ButtonBox.ButtonBoxChild)(this.hbuttonbox[this.buttonUp]));
			w5.Position = 2;
			w5.Expand = false;
			w5.Fill = false;
			// Container child hbuttonbox.Gtk.ButtonBox+ButtonBoxChild
			this.buttonDown = new global::Gtk.Button ();
			this.buttonDown.Sensitive = false;
			this.buttonDown.CanFocus = true;
			this.buttonDown.Name = "buttonDown";
			this.buttonDown.UseStock = true;
			this.buttonDown.UseUnderline = true;
			this.buttonDown.Label = "gtk-go-down";
			this.hbuttonbox.Add (this.buttonDown);
			global::Gtk.ButtonBox.ButtonBoxChild w6 = ((global::Gtk.ButtonBox.ButtonBoxChild)(this.hbuttonbox[this.buttonDown]));
			w6.Position = 3;
			w6.Expand = false;
			w6.Fill = false;
			this.vboxLeft.Add (this.hbuttonbox);
			global::Gtk.Box.BoxChild w7 = ((global::Gtk.Box.BoxChild)(this.vboxLeft[this.hbuttonbox]));
			w7.Position = 1;
			w7.Expand = false;
			w7.Fill = false;
			this.hpaned.Add (this.vboxLeft);
			global::Gtk.Paned.PanedChild w8 = ((global::Gtk.Paned.PanedChild)(this.hpaned[this.vboxLeft]));
			w8.Resize = false;
			// Container child hpaned.Gtk.Paned+PanedChild
			this.scrolledwindowRight = new global::Gtk.ScrolledWindow ();
			this.scrolledwindowRight.CanFocus = true;
			this.scrolledwindowRight.Name = "scrolledwindowRight";
			this.scrolledwindowRight.ShadowType = ((global::Gtk.ShadowType)(1));
			this.hpaned.Add (this.scrolledwindowRight);
			this.Add (this.hpaned);
			if ((this.Child != null)) {
				this.Child.ShowAll ();
			}
			this.Show ();
			this.buttonAdd.Clicked += new global::System.EventHandler (this.AddClicked);
			this.buttonRemove.Clicked += new global::System.EventHandler (this.RemoveClicked);
			this.buttonUp.Clicked += new global::System.EventHandler (this.UpClicked);
			this.buttonDown.Clicked += new global::System.EventHandler (this.DownClicked);
		}
	}
}
