
// This file has been generated by the GUI designer. Do not modify.
namespace MonoDevelop.Database.Designer
{
	public partial class ProcedureEditorDialog
	{
		private global::Gtk.VBox vboxContent;

		private global::Gtk.HBox hboxName;

		private global::Gtk.Label nameLabel;

		private global::Gtk.Entry entryName;

		private global::Gtk.HBox hboxWarning;

		private global::Gtk.Image imageWarning;

		private global::Gtk.Label labelWarning;

		private global::Gtk.Button buttonOpen;

		private global::Gtk.Button buttonSave;

		private global::Gtk.Button buttonCancel;

		private global::Gtk.Button buttonOk;

		protected virtual void Build ()
		{
			global::Stetic.Gui.Initialize (this);
			// Widget MonoDevelop.Database.Designer.ProcedureEditorDialog
			this.Name = "MonoDevelop.Database.Designer.ProcedureEditorDialog";
			this.WindowPosition = ((global::Gtk.WindowPosition)(4));
			// Internal child MonoDevelop.Database.Designer.ProcedureEditorDialog.VBox
			global::Gtk.VBox w1 = this.VBox;
			w1.Name = "dialog1_VBox";
			w1.BorderWidth = ((uint)(2));
			// Container child dialog1_VBox.Gtk.Box+BoxChild
			this.vboxContent = new global::Gtk.VBox ();
			this.vboxContent.Name = "vboxContent";
			this.vboxContent.Spacing = 6;
			this.vboxContent.BorderWidth = ((uint)(6));
			// Container child vboxContent.Gtk.Box+BoxChild
			this.hboxName = new global::Gtk.HBox ();
			this.hboxName.Name = "hboxName";
			this.hboxName.Spacing = 6;
			// Container child hboxName.Gtk.Box+BoxChild
			this.nameLabel = new global::Gtk.Label ();
			this.nameLabel.Name = "nameLabel";
			this.nameLabel.Xalign = 0f;
			this.nameLabel.LabelProp = global::MonoDevelop.Database.AddinCatalog.GetString ("Name");
			this.hboxName.Add (this.nameLabel);
			global::Gtk.Box.BoxChild w2 = ((global::Gtk.Box.BoxChild)(this.hboxName[this.nameLabel]));
			w2.Position = 0;
			w2.Expand = false;
			w2.Fill = false;
			// Container child hboxName.Gtk.Box+BoxChild
			this.entryName = new global::Gtk.Entry ();
			this.entryName.CanFocus = true;
			this.entryName.Name = "entryName";
			this.entryName.IsEditable = true;
			this.entryName.InvisibleChar = '●';
			this.hboxName.Add (this.entryName);
			global::Gtk.Box.BoxChild w3 = ((global::Gtk.Box.BoxChild)(this.hboxName[this.entryName]));
			w3.Position = 1;
			this.vboxContent.Add (this.hboxName);
			global::Gtk.Box.BoxChild w4 = ((global::Gtk.Box.BoxChild)(this.vboxContent[this.hboxName]));
			w4.Position = 0;
			w4.Expand = false;
			w4.Fill = false;
			// Container child vboxContent.Gtk.Box+BoxChild
			this.hboxWarning = new global::Gtk.HBox ();
			this.hboxWarning.Name = "hboxWarning";
			this.hboxWarning.Spacing = 6;
			// Container child hboxWarning.Gtk.Box+BoxChild
			this.imageWarning = new global::Gtk.Image ();
			this.imageWarning.Name = "imageWarning";
			this.imageWarning.Pixbuf = global::Stetic.IconLoader.LoadIcon (this, "gtk-dialog-warning", global::Gtk.IconSize.Menu);
			this.hboxWarning.Add (this.imageWarning);
			global::Gtk.Box.BoxChild w5 = ((global::Gtk.Box.BoxChild)(this.hboxWarning[this.imageWarning]));
			w5.Position = 0;
			w5.Expand = false;
			w5.Fill = false;
			// Container child hboxWarning.Gtk.Box+BoxChild
			this.labelWarning = new global::Gtk.Label ();
			this.labelWarning.Name = "labelWarning";
			this.labelWarning.Xalign = 0f;
			this.hboxWarning.Add (this.labelWarning);
			global::Gtk.Box.BoxChild w6 = ((global::Gtk.Box.BoxChild)(this.hboxWarning[this.labelWarning]));
			w6.Position = 1;
			this.vboxContent.Add (this.hboxWarning);
			global::Gtk.Box.BoxChild w7 = ((global::Gtk.Box.BoxChild)(this.vboxContent[this.hboxWarning]));
			w7.Position = 2;
			w7.Expand = false;
			w7.Fill = false;
			w1.Add (this.vboxContent);
			global::Gtk.Box.BoxChild w8 = ((global::Gtk.Box.BoxChild)(w1[this.vboxContent]));
			w8.Position = 0;
			// Internal child MonoDevelop.Database.Designer.ProcedureEditorDialog.ActionArea
			global::Gtk.HButtonBox w9 = this.ActionArea;
			w9.Name = "dialog1_ActionArea";
			w9.Spacing = 6;
			w9.BorderWidth = ((uint)(5));
			w9.LayoutStyle = ((global::Gtk.ButtonBoxStyle)(4));
			// Container child dialog1_ActionArea.Gtk.ButtonBox+ButtonBoxChild
			this.buttonOpen = new global::Gtk.Button ();
			this.buttonOpen.CanFocus = true;
			this.buttonOpen.Name = "buttonOpen";
			this.buttonOpen.UseUnderline = true;
			// Container child buttonOpen.Gtk.Container+ContainerChild
			global::Gtk.Alignment w10 = new global::Gtk.Alignment (0.5f, 0.5f, 0f, 0f);
			// Container child GtkAlignment.Gtk.Container+ContainerChild
			global::Gtk.HBox w11 = new global::Gtk.HBox ();
			w11.Spacing = 2;
			// Container child GtkHBox.Gtk.Container+ContainerChild
			global::Gtk.Image w12 = new global::Gtk.Image ();
			w12.Pixbuf = global::Stetic.IconLoader.LoadIcon (this, "gtk-open", global::Gtk.IconSize.Menu);
			w11.Add (w12);
			// Container child GtkHBox.Gtk.Container+ContainerChild
			global::Gtk.Label w14 = new global::Gtk.Label ();
			w14.LabelProp = global::MonoDevelop.Database.AddinCatalog.GetString ("_Open Script");
			w14.UseUnderline = true;
			w11.Add (w14);
			w10.Add (w11);
			this.buttonOpen.Add (w10);
			this.AddActionWidget (this.buttonOpen, -3);
			global::Gtk.ButtonBox.ButtonBoxChild w18 = ((global::Gtk.ButtonBox.ButtonBoxChild)(w9[this.buttonOpen]));
			w18.Expand = false;
			w18.Fill = false;
			// Container child dialog1_ActionArea.Gtk.ButtonBox+ButtonBoxChild
			this.buttonSave = new global::Gtk.Button ();
			this.buttonSave.CanFocus = true;
			this.buttonSave.Name = "buttonSave";
			this.buttonSave.UseUnderline = true;
			// Container child buttonSave.Gtk.Container+ContainerChild
			global::Gtk.Alignment w19 = new global::Gtk.Alignment (0.5f, 0.5f, 0f, 0f);
			// Container child GtkAlignment.Gtk.Container+ContainerChild
			global::Gtk.HBox w20 = new global::Gtk.HBox ();
			w20.Spacing = 2;
			// Container child GtkHBox.Gtk.Container+ContainerChild
			global::Gtk.Image w21 = new global::Gtk.Image ();
			w21.Pixbuf = global::Stetic.IconLoader.LoadIcon (this, "gtk-save", global::Gtk.IconSize.Menu);
			w20.Add (w21);
			// Container child GtkHBox.Gtk.Container+ContainerChild
			global::Gtk.Label w23 = new global::Gtk.Label ();
			w23.LabelProp = global::MonoDevelop.Database.AddinCatalog.GetString ("_Save Script");
			w23.UseUnderline = true;
			w20.Add (w23);
			w19.Add (w20);
			this.buttonSave.Add (w19);
			this.AddActionWidget (this.buttonSave, -3);
			global::Gtk.ButtonBox.ButtonBoxChild w27 = ((global::Gtk.ButtonBox.ButtonBoxChild)(w9[this.buttonSave]));
			w27.Position = 1;
			w27.Expand = false;
			w27.Fill = false;
			// Container child dialog1_ActionArea.Gtk.ButtonBox+ButtonBoxChild
			this.buttonCancel = new global::Gtk.Button ();
			this.buttonCancel.CanDefault = true;
			this.buttonCancel.CanFocus = true;
			this.buttonCancel.Name = "buttonCancel";
			this.buttonCancel.UseStock = true;
			this.buttonCancel.UseUnderline = true;
			this.buttonCancel.Label = "gtk-cancel";
			this.AddActionWidget (this.buttonCancel, -6);
			global::Gtk.ButtonBox.ButtonBoxChild w28 = ((global::Gtk.ButtonBox.ButtonBoxChild)(w9[this.buttonCancel]));
			w28.Position = 2;
			w28.Expand = false;
			w28.Fill = false;
			// Container child dialog1_ActionArea.Gtk.ButtonBox+ButtonBoxChild
			this.buttonOk = new global::Gtk.Button ();
			this.buttonOk.Sensitive = false;
			this.buttonOk.CanDefault = true;
			this.buttonOk.CanFocus = true;
			this.buttonOk.Name = "buttonOk";
			this.buttonOk.UseStock = true;
			this.buttonOk.UseUnderline = true;
			this.buttonOk.Label = "gtk-ok";
			this.AddActionWidget (this.buttonOk, -5);
			global::Gtk.ButtonBox.ButtonBoxChild w29 = ((global::Gtk.ButtonBox.ButtonBoxChild)(w9[this.buttonOk]));
			w29.Position = 3;
			w29.Expand = false;
			w29.Fill = false;
			if ((this.Child != null)) {
				this.Child.ShowAll ();
			}
			this.DefaultWidth = 640;
			this.DefaultHeight = 480;
			this.Show ();
			this.entryName.Changed += new global::System.EventHandler (this.NameChanged);
			this.buttonOpen.Clicked += new global::System.EventHandler (this.OnButtonOpenClicked);
			this.buttonSave.Clicked += new global::System.EventHandler (this.OnButtonSaveClicked);
			this.buttonCancel.Clicked += new global::System.EventHandler (this.CancelClicked);
			this.buttonOk.Clicked += new global::System.EventHandler (this.OkClicked);
		}
	}
}
