// Autogenerated from MonoDevelop.Database.Sql.Sqlite.addin.xml

using System.Reflection;

[assembly: AssemblyProduct ("MonoDevelop")]
[assembly: AssemblyTitle ("Sqlite provider for Mono Data Sql")]
[assembly: AssemblyDescription ("Database Module")]
[assembly: AssemblyVersion ("2.6")]
[assembly: AssemblyCopyright ("MIT X11")]
