
// This file has been generated by the GUI designer. Do not modify.
namespace MonoDevelop.Database.Components
{
	public partial class DataGrid
	{
		private global::Gtk.VBox vbox;

		private global::Gtk.ScrolledWindow scrolledwindow;

		private global::Gtk.TreeView grid;

		private global::Gtk.HBox hbox;

		private global::Gtk.Button buttonFirst;

		private global::Gtk.Button buttonPrevious;

		private global::Gtk.Entry entryCurrent;

		private global::Gtk.Label label1;

		private global::Gtk.Entry entryTotal;

		private global::Gtk.Button buttonNext;

		private global::Gtk.Button buttonLast;

		protected virtual void Build ()
		{
			global::Stetic.Gui.Initialize (this);
			// Widget MonoDevelop.Database.Components.DataGrid
			global::Stetic.BinContainer.Attach (this);
			this.Name = "MonoDevelop.Database.Components.DataGrid";
			// Container child MonoDevelop.Database.Components.DataGrid.Gtk.Container+ContainerChild
			this.vbox = new global::Gtk.VBox ();
			this.vbox.Name = "vbox";
			this.vbox.Spacing = 5;
			this.vbox.BorderWidth = ((uint)(5));
			// Container child vbox.Gtk.Box+BoxChild
			this.scrolledwindow = new global::Gtk.ScrolledWindow ();
			this.scrolledwindow.CanFocus = true;
			this.scrolledwindow.Name = "scrolledwindow";
			this.scrolledwindow.ShadowType = ((global::Gtk.ShadowType)(1));
			// Container child scrolledwindow.Gtk.Container+ContainerChild
			this.grid = new global::Gtk.TreeView ();
			this.grid.CanDefault = true;
			this.grid.CanFocus = true;
			this.grid.Name = "grid";
			this.grid.EnableSearch = false;
			this.scrolledwindow.Add (this.grid);
			this.vbox.Add (this.scrolledwindow);
			global::Gtk.Box.BoxChild w2 = ((global::Gtk.Box.BoxChild)(this.vbox[this.scrolledwindow]));
			w2.Position = 0;
			// Container child vbox.Gtk.Box+BoxChild
			this.hbox = new global::Gtk.HBox ();
			this.hbox.Name = "hbox";
			this.hbox.Spacing = 5;
			// Container child hbox.Gtk.Box+BoxChild
			this.buttonFirst = new global::Gtk.Button ();
			this.buttonFirst.CanFocus = true;
			this.buttonFirst.Name = "buttonFirst";
			this.buttonFirst.UseStock = true;
			this.buttonFirst.UseUnderline = true;
			this.buttonFirst.Label = "gtk-goto-first";
			this.hbox.Add (this.buttonFirst);
			global::Gtk.Box.BoxChild w3 = ((global::Gtk.Box.BoxChild)(this.hbox[this.buttonFirst]));
			w3.Position = 0;
			w3.Expand = false;
			w3.Fill = false;
			// Container child hbox.Gtk.Box+BoxChild
			this.buttonPrevious = new global::Gtk.Button ();
			this.buttonPrevious.CanFocus = true;
			this.buttonPrevious.Name = "buttonPrevious";
			this.buttonPrevious.UseStock = true;
			this.buttonPrevious.UseUnderline = true;
			this.buttonPrevious.Label = "gtk-go-back";
			this.hbox.Add (this.buttonPrevious);
			global::Gtk.Box.BoxChild w4 = ((global::Gtk.Box.BoxChild)(this.hbox[this.buttonPrevious]));
			w4.Position = 1;
			w4.Expand = false;
			w4.Fill = false;
			// Container child hbox.Gtk.Box+BoxChild
			this.entryCurrent = new global::Gtk.Entry ();
			this.entryCurrent.CanFocus = true;
			this.entryCurrent.Name = "entryCurrent";
			this.entryCurrent.IsEditable = true;
			this.entryCurrent.InvisibleChar = '●';
			this.hbox.Add (this.entryCurrent);
			global::Gtk.Box.BoxChild w5 = ((global::Gtk.Box.BoxChild)(this.hbox[this.entryCurrent]));
			w5.Position = 2;
			w5.Expand = false;
			// Container child hbox.Gtk.Box+BoxChild
			this.label1 = new global::Gtk.Label ();
			this.label1.Name = "label1";
			this.label1.LabelProp = global::MonoDevelop.Database.AddinCatalog.GetString ("of");
			this.hbox.Add (this.label1);
			global::Gtk.Box.BoxChild w6 = ((global::Gtk.Box.BoxChild)(this.hbox[this.label1]));
			w6.Position = 3;
			w6.Expand = false;
			w6.Fill = false;
			// Container child hbox.Gtk.Box+BoxChild
			this.entryTotal = new global::Gtk.Entry ();
			this.entryTotal.CanFocus = true;
			this.entryTotal.Name = "entryTotal";
			this.entryTotal.IsEditable = false;
			this.entryTotal.InvisibleChar = '●';
			this.hbox.Add (this.entryTotal);
			global::Gtk.Box.BoxChild w7 = ((global::Gtk.Box.BoxChild)(this.hbox[this.entryTotal]));
			w7.Position = 4;
			w7.Expand = false;
			// Container child hbox.Gtk.Box+BoxChild
			this.buttonNext = new global::Gtk.Button ();
			this.buttonNext.CanFocus = true;
			this.buttonNext.Name = "buttonNext";
			this.buttonNext.UseStock = true;
			this.buttonNext.UseUnderline = true;
			this.buttonNext.Label = "gtk-go-forward";
			this.hbox.Add (this.buttonNext);
			global::Gtk.Box.BoxChild w8 = ((global::Gtk.Box.BoxChild)(this.hbox[this.buttonNext]));
			w8.Position = 5;
			w8.Expand = false;
			w8.Fill = false;
			// Container child hbox.Gtk.Box+BoxChild
			this.buttonLast = new global::Gtk.Button ();
			this.buttonLast.CanFocus = true;
			this.buttonLast.Name = "buttonLast";
			this.buttonLast.UseStock = true;
			this.buttonLast.UseUnderline = true;
			this.buttonLast.Label = "gtk-goto-last";
			this.hbox.Add (this.buttonLast);
			global::Gtk.Box.BoxChild w9 = ((global::Gtk.Box.BoxChild)(this.hbox[this.buttonLast]));
			w9.Position = 6;
			w9.Expand = false;
			w9.Fill = false;
			this.vbox.Add (this.hbox);
			global::Gtk.Box.BoxChild w10 = ((global::Gtk.Box.BoxChild)(this.vbox[this.hbox]));
			w10.Position = 1;
			w10.Expand = false;
			w10.Fill = false;
			this.Add (this.vbox);
			if ((this.Child != null)) {
				this.Child.ShowAll ();
			}
			this.Show ();
			this.buttonFirst.Clicked += new global::System.EventHandler (this.ButtonFirstClicked);
			this.buttonPrevious.Clicked += new global::System.EventHandler (this.ButtonPreviousClicked);
			this.entryCurrent.Activated += new global::System.EventHandler (this.EntryCurrentActivated);
			this.buttonNext.Clicked += new global::System.EventHandler (this.ButtonNextClicked);
			this.buttonLast.Clicked += new global::System.EventHandler (this.ButtonLastClicked);
		}
	}
}
