
// This file has been generated by the GUI designer. Do not modify.
namespace MonoDevelop.Database.Sql.Npgsql
{
	public partial class NpgsqlCreateDatabaseWidget
	{
		private global::Gtk.Table table1;

		private global::Gtk.ComboBoxEntry comboEncoding;

		private global::Gtk.ComboBoxEntry comboOwner;

		private global::Gtk.ComboBoxEntry comboTablespace;

		private global::Gtk.ComboBoxEntry comboTemplate;

		private global::Gtk.Label labelEncoding;

		private global::Gtk.Label labelOwner;

		private global::Gtk.Label labelTablespace;

		private global::Gtk.Label labelTemplate;

		protected virtual void Build ()
		{
			global::Stetic.Gui.Initialize (this);
			// Widget MonoDevelop.Database.Sql.Npgsql.NpgsqlCreateDatabaseWidget
			global::Stetic.BinContainer.Attach (this);
			this.Name = "MonoDevelop.Database.Sql.Npgsql.NpgsqlCreateDatabaseWidget";
			// Container child MonoDevelop.Database.Sql.Npgsql.NpgsqlCreateDatabaseWidget.Gtk.Container+ContainerChild
			this.table1 = new global::Gtk.Table (((uint)(4)), ((uint)(2)), false);
			this.table1.Name = "table1";
			this.table1.RowSpacing = ((uint)(6));
			this.table1.ColumnSpacing = ((uint)(6));
			// Container child table1.Gtk.Table+TableChild
			this.comboEncoding = new global::Gtk.ComboBoxEntry ();
			this.comboEncoding.Name = "comboEncoding";
			this.table1.Add (this.comboEncoding);
			global::Gtk.Table.TableChild w1 = ((global::Gtk.Table.TableChild)(this.table1[this.comboEncoding]));
			w1.TopAttach = ((uint)(2));
			w1.BottomAttach = ((uint)(3));
			w1.LeftAttach = ((uint)(1));
			w1.RightAttach = ((uint)(2));
			w1.XOptions = ((global::Gtk.AttachOptions)(4));
			w1.YOptions = ((global::Gtk.AttachOptions)(4));
			// Container child table1.Gtk.Table+TableChild
			this.comboOwner = new global::Gtk.ComboBoxEntry ();
			this.comboOwner.Name = "comboOwner";
			this.table1.Add (this.comboOwner);
			global::Gtk.Table.TableChild w2 = ((global::Gtk.Table.TableChild)(this.table1[this.comboOwner]));
			w2.LeftAttach = ((uint)(1));
			w2.RightAttach = ((uint)(2));
			w2.YOptions = ((global::Gtk.AttachOptions)(4));
			// Container child table1.Gtk.Table+TableChild
			this.comboTablespace = new global::Gtk.ComboBoxEntry ();
			this.comboTablespace.Name = "comboTablespace";
			this.table1.Add (this.comboTablespace);
			global::Gtk.Table.TableChild w3 = ((global::Gtk.Table.TableChild)(this.table1[this.comboTablespace]));
			w3.TopAttach = ((uint)(3));
			w3.BottomAttach = ((uint)(4));
			w3.LeftAttach = ((uint)(1));
			w3.RightAttach = ((uint)(2));
			w3.XOptions = ((global::Gtk.AttachOptions)(4));
			w3.YOptions = ((global::Gtk.AttachOptions)(4));
			// Container child table1.Gtk.Table+TableChild
			this.comboTemplate = new global::Gtk.ComboBoxEntry ();
			this.comboTemplate.Name = "comboTemplate";
			this.table1.Add (this.comboTemplate);
			global::Gtk.Table.TableChild w4 = ((global::Gtk.Table.TableChild)(this.table1[this.comboTemplate]));
			w4.TopAttach = ((uint)(1));
			w4.BottomAttach = ((uint)(2));
			w4.LeftAttach = ((uint)(1));
			w4.RightAttach = ((uint)(2));
			w4.XOptions = ((global::Gtk.AttachOptions)(4));
			w4.YOptions = ((global::Gtk.AttachOptions)(4));
			// Container child table1.Gtk.Table+TableChild
			this.labelEncoding = new global::Gtk.Label ();
			this.labelEncoding.Name = "labelEncoding";
			this.labelEncoding.LabelProp = global::MonoDevelop.Database.AddinCatalog.GetString ("Encoding");
			this.table1.Add (this.labelEncoding);
			global::Gtk.Table.TableChild w5 = ((global::Gtk.Table.TableChild)(this.table1[this.labelEncoding]));
			w5.TopAttach = ((uint)(2));
			w5.BottomAttach = ((uint)(3));
			w5.XOptions = ((global::Gtk.AttachOptions)(4));
			w5.YOptions = ((global::Gtk.AttachOptions)(4));
			// Container child table1.Gtk.Table+TableChild
			this.labelOwner = new global::Gtk.Label ();
			this.labelOwner.Name = "labelOwner";
			this.labelOwner.LabelProp = global::MonoDevelop.Database.AddinCatalog.GetString ("Owner");
			this.table1.Add (this.labelOwner);
			global::Gtk.Table.TableChild w6 = ((global::Gtk.Table.TableChild)(this.table1[this.labelOwner]));
			w6.XOptions = ((global::Gtk.AttachOptions)(4));
			w6.YOptions = ((global::Gtk.AttachOptions)(4));
			// Container child table1.Gtk.Table+TableChild
			this.labelTablespace = new global::Gtk.Label ();
			this.labelTablespace.Name = "labelTablespace";
			this.labelTablespace.LabelProp = global::MonoDevelop.Database.AddinCatalog.GetString ("Tablespace");
			this.table1.Add (this.labelTablespace);
			global::Gtk.Table.TableChild w7 = ((global::Gtk.Table.TableChild)(this.table1[this.labelTablespace]));
			w7.TopAttach = ((uint)(3));
			w7.BottomAttach = ((uint)(4));
			w7.XOptions = ((global::Gtk.AttachOptions)(4));
			w7.YOptions = ((global::Gtk.AttachOptions)(4));
			// Container child table1.Gtk.Table+TableChild
			this.labelTemplate = new global::Gtk.Label ();
			this.labelTemplate.Name = "labelTemplate";
			this.labelTemplate.LabelProp = global::MonoDevelop.Database.AddinCatalog.GetString ("Template");
			this.table1.Add (this.labelTemplate);
			global::Gtk.Table.TableChild w8 = ((global::Gtk.Table.TableChild)(this.table1[this.labelTemplate]));
			w8.TopAttach = ((uint)(1));
			w8.BottomAttach = ((uint)(2));
			w8.XOptions = ((global::Gtk.AttachOptions)(4));
			w8.YOptions = ((global::Gtk.AttachOptions)(4));
			this.Add (this.table1);
			if ((this.Child != null)) {
				this.Child.ShowAll ();
			}
			this.Show ();
		}
	}
}
