
// This file has been generated by the GUI designer. Do not modify.
namespace MonoDevelop.Database.Designer
{
	public partial class CreateDatabaseDialog
	{
		private global::Gtk.Notebook notebook;

		private global::Gtk.VBox vboxConnection;

		private global::Gtk.Label label1;

		private global::Gtk.Button buttonCancel;

		private global::Gtk.Button buttonOk;

		protected virtual void Build ()
		{
			global::Stetic.Gui.Initialize (this);
			// Widget MonoDevelop.Database.Designer.CreateDatabaseDialog
			this.Name = "MonoDevelop.Database.Designer.CreateDatabaseDialog";
			this.Title = global::MonoDevelop.Database.AddinCatalog.GetString ("Create Database");
			this.TypeHint = ((global::Gdk.WindowTypeHint)(1));
			this.WindowPosition = ((global::Gtk.WindowPosition)(1));
			this.SkipTaskbarHint = true;
			// Internal child MonoDevelop.Database.Designer.CreateDatabaseDialog.VBox
			global::Gtk.VBox w1 = this.VBox;
			w1.Name = "dialog1_VBox";
			w1.BorderWidth = ((uint)(2));
			// Container child dialog1_VBox.Gtk.Box+BoxChild
			this.notebook = new global::Gtk.Notebook ();
			this.notebook.CanFocus = true;
			this.notebook.Name = "notebook";
			this.notebook.CurrentPage = 0;
			this.notebook.ShowTabs = false;
			// Container child notebook.Gtk.Notebook+NotebookChild
			this.vboxConnection = new global::Gtk.VBox ();
			this.vboxConnection.Name = "vboxConnection";
			this.vboxConnection.Spacing = 6;
			this.vboxConnection.BorderWidth = ((uint)(6));
			this.notebook.Add (this.vboxConnection);
			// Notebook tab
			this.label1 = new global::Gtk.Label ();
			this.label1.Name = "label1";
			this.label1.LabelProp = global::MonoDevelop.Database.AddinCatalog.GetString ("Connection");
			this.notebook.SetTabLabel (this.vboxConnection, this.label1);
			this.label1.ShowAll ();
			w1.Add (this.notebook);
			global::Gtk.Box.BoxChild w3 = ((global::Gtk.Box.BoxChild)(w1[this.notebook]));
			w3.Position = 0;
			// Internal child MonoDevelop.Database.Designer.CreateDatabaseDialog.ActionArea
			global::Gtk.HButtonBox w4 = this.ActionArea;
			w4.Name = "dialog1_ActionArea";
			w4.Spacing = 6;
			w4.BorderWidth = ((uint)(5));
			w4.LayoutStyle = ((global::Gtk.ButtonBoxStyle)(4));
			// Container child dialog1_ActionArea.Gtk.ButtonBox+ButtonBoxChild
			this.buttonCancel = new global::Gtk.Button ();
			this.buttonCancel.CanDefault = true;
			this.buttonCancel.CanFocus = true;
			this.buttonCancel.Name = "buttonCancel";
			this.buttonCancel.UseStock = true;
			this.buttonCancel.UseUnderline = true;
			this.buttonCancel.Label = "gtk-cancel";
			this.AddActionWidget (this.buttonCancel, -6);
			global::Gtk.ButtonBox.ButtonBoxChild w5 = ((global::Gtk.ButtonBox.ButtonBoxChild)(w4[this.buttonCancel]));
			w5.Expand = false;
			w5.Fill = false;
			// Container child dialog1_ActionArea.Gtk.ButtonBox+ButtonBoxChild
			this.buttonOk = new global::Gtk.Button ();
			this.buttonOk.Sensitive = false;
			this.buttonOk.CanDefault = true;
			this.buttonOk.CanFocus = true;
			this.buttonOk.Name = "buttonOk";
			this.buttonOk.UseStock = true;
			this.buttonOk.UseUnderline = true;
			this.buttonOk.Label = "gtk-ok";
			this.AddActionWidget (this.buttonOk, -5);
			global::Gtk.ButtonBox.ButtonBoxChild w6 = ((global::Gtk.ButtonBox.ButtonBoxChild)(w4[this.buttonOk]));
			w6.Position = 1;
			w6.Expand = false;
			w6.Fill = false;
			if ((this.Child != null)) {
				this.Child.ShowAll ();
			}
			this.DefaultWidth = 662;
			this.DefaultHeight = 474;
			this.Show ();
			this.buttonCancel.Clicked += new global::System.EventHandler (this.CancelClicked);
			this.buttonOk.Clicked += new global::System.EventHandler (this.OkClicked);
		}
	}
}
